@@ -9,7 +9,7 @@ import numpy as np
 import param
 
 from ..dimension import redim
-from .interface import Interface
+from .interface import Interface, iloc, ndloc
 from .array import ArrayInterface
 from .dictionary import DictInterface
 from .grid import GridInterface
@@ -433,7 +433,9 @@ class Dataset(Element):
             else:
                 selection = tuple(selection.columns(kdims+self.vdims).values())
 
-            return self.clone(selection, kdims=kdims, new_type=new_type)
+            datatype = list(util.unique_iterator(self.datatype+['dataframe', 'dict']))
+            return self.clone(selection, kdims=kdims, new_type=new_type,
+                              datatype=datatype)
 
         lens = set(len(util.wrap_tuple(s)) for s in samples)
         if len(lens) > 1:
@@ -472,6 +474,14 @@ class Dataset(Element):
         if dimensions is None: dimensions = self.kdims
         elif not isinstance(dimensions, list): dimensions = [dimensions]
         kdims = [self.get_dimension(d, strict=True) for d in dimensions]
+        if not len(self):
+            if spreadfn:
+                spread_name = spreadfn.__name__
+                vdims = [d for vd in self.vdims for d in [vd, vd('_'.join([vd.name, spread_name]))]]
+            else:
+                vdims = self.vdims
+            return self.clone([], kdims=kdims, vdims=vdims)
+
         aggregated = self.interface.aggregate(self, kdims, function, **kwargs)
         aggregated = self.interface.unpack_scalar(self, aggregated)
 
@@ -488,7 +498,7 @@ class Dataset(Element):
             combined = self.clone(aggregated, kdims=kdims, new_type=Dataset)
             for i, d in enumerate(vdims):
                 dim = d('_'.join([d.name, spread_name]))
-                dvals = error.dimension_values(d, False, False)
+                dvals = error.dimension_values(d, flat=False)
                 combined = combined.add_dimension(dim, ndims+i, dvals, True)
             return combined.clone(new_type=Dataset if generic_type else type(self))
 
@@ -616,6 +626,59 @@ class Dataset(Element):
         return self._conversion_interface(self)
 
 
+    @property
+    def iloc(self):
+        """
+        Returns an iloc object providing a convenient interface to
+        slice and index into the Dataset using row and column indices.
+        Allow selection by integer index, slice and list of integer
+        indices and boolean arrays.
+
+        Examples:
+
+        * Index the first row and column:
+
+            dataset.iloc[0, 0]
+
+        * Select rows 1 and 2 with a slice:
+
+            dataset.iloc[1:3, :]
+
+        * Select with a list of integer coordinates:
+
+            dataset.iloc[[0, 2, 3]]
+        """
+        return iloc(self)
+
+
+    @property
+    def ndloc(self):
+        """
+        Returns an ndloc object providing nd-array like indexing for
+        gridded datasets. Follows NumPy array indexing conventions,
+        allowing for indexing, slicing and selecting a list of indices
+        on multi-dimensional arrays using integer indices. The order
+        of array indices is inverted relative to the Dataset key
+        dimensions, e.g. an Image with key dimensions 'x' and 'y' can
+        be indexed with ``image.ndloc[iy, ix]``, where ``iy`` and
+        ``ix`` are integer indices along the y and x dimensions.
+
+        Examples:
+
+        * Index value in 2D array:
+
+            dataset.ndloc[3, 1]
+
+        * Slice along y-axis of 2D array:
+
+            dataset.ndloc[2:5, :]
+
+        * Vectorized (non-orthogonal) indexing along x- and y-axes:
+
+            dataset.ndloc[[1, 2, 3], [0, 2, 3]]
+        """
+        return ndloc(self)
+
 
 # Aliases for pickle backward compatibility
 Columns      = Dataset
