@@ -14,7 +14,8 @@ import param
 from ..core.util import (basestring, sanitize_identifier,
                          group_sanitizer, label_sanitizer, max_range,
                          find_range, dimension_sanitizer, OrderedDict,
-                         bytes_to_unicode, unicode, dt64_to_dt, unique_array)
+                         bytes_to_unicode, unicode, dt64_to_dt, unique_array,
+                         builtins, config)
 from .options import Store, StoreOptions
 from .pprint import PrettyPrinter
 
@@ -157,6 +158,13 @@ class redim(object):
     def range(self, specs=None, **values):
         return self._redim('range', specs, **values)
 
+    def label(self, specs=None, **values):
+        for k, v in values.items():
+            dim = self.parent.get_dimension(k)
+            if dim and dim.name != dim.label and dim.label != v:
+                raise ValueError('Cannot override an existing Dimension label')
+        return self._redim('label', specs, **values)
+
     def soft_range(self, specs=None, **values):
         return self._redim('soft_range', specs, **values)
 
@@ -297,8 +305,9 @@ class Dimension(param.Parameterized):
             all_params['name'] = name
             all_params['label'] = label
             if 'label' in params and (label != params['label']):
-                self.warning('Using label as supplied by keyword ({!r}), ignoring '
-                             'tuple value {!r}'.format(params['label'], label))
+                if params['label'] != label:
+                    self.warning('Using label as supplied by keyword ({!r}), ignoring '
+                                 'tuple value {!r}'.format(params['label'], label))
                 all_params['label'] = params['label']
         elif isinstance(spec, basestring):
             all_params['name'] = spec
@@ -314,7 +323,7 @@ class Dimension(param.Parameterized):
             self.warning("The 'initial' string for dimension values is no longer supported.")
             values = []
 
-        all_params['values'] = sorted(list(unique_array(values)))
+        all_params['values'] = list(unique_array(values))
         super(Dimension, self).__init__(**all_params)
 
 
@@ -341,7 +350,8 @@ class Dimension(param.Parameterized):
         if 'label' in overrides and isinstance(spec, basestring) :
             spec = (spec, overrides['label'])
         elif 'label' in overrides and isinstance(spec, tuple) :
-            self.warning('Using label as supplied by keyword ({!r}), ignoring '
+            if overrides['label'] != spec[1]:
+                self.warning('Using label as supplied by keyword ({!r}), ignoring '
                              'tuple value {!r}'.format(overrides['label'], spec[1]))
             spec = (spec[0],  overrides['label'])
 
@@ -479,7 +489,7 @@ class LabelledData(param.Parameterized):
 
     _deep_indexable = False
 
-    def __init__(self, data, id=None, **params):
+    def __init__(self, data, id=None, plot_id=None, **params):
         """
         All LabelledData subclasses must supply data to the
         constructor, which will be held on the .data attribute.
@@ -488,6 +498,7 @@ class LabelledData(param.Parameterized):
         """
         self.data = data
         self.id = id
+        self._plot_id = plot_id or builtins.id(self)
         if isinstance(params.get('label',None), tuple):
             (alias, long_name) = params['label']
             label_sanitizer.add_aliases(**{alias:long_name})
@@ -531,6 +542,7 @@ class LabelledData(param.Parameterized):
 
         if data is None and shared_data:
             data = self.data
+            settings['plot_id'] = self._plot_id
         # Apply name mangling for __ attribute
         pos_args = getattr(self, '_' + type(self).__name__ + '__pos_params', [])
         return clone_type(data, *args, **{k:v for k,v in settings.items()
@@ -1065,11 +1077,7 @@ class Dimensioned(LabelledData):
 
 
     def __repr__(self):
-        reprval = PrettyPrinter.pprint(self)
-        if isinstance(reprval, unicode):
-            return str(reprval.encode("utf8"))
-        else:
-            return str(reprval)
+        return PrettyPrinter.pprint(self)
 
     def __str__(self):
         return repr(self)
@@ -1078,13 +1086,27 @@ class Dimensioned(LabelledData):
         return unicode(PrettyPrinter.pprint(self))
 
 
-
     def __call__(self, options=None, **kwargs):
+        if config.warn_options_call:
+            self.warning('Use of __call__ to set options will be deprecated '
+                         'in future. Use the equivalent opts method instead.')
+        return self.opts(options, **kwargs)
+
+    def opts(self, options=None, **kwargs):
         """
         Apply the supplied options to a clone of the object which is
         then returned. Note that if no options are supplied at all,
         all ids are reset.
         """
+        from ..util.parser import OptsSpec
+        if isinstance(options, basestring):
+            try:
+                options = OptsSpec.parse(options)
+            except SyntaxError:
+                options = OptsSpec.parse(
+                    '{clsname} {options}'.format(clsname=self.__class__.__name__,
+                                                 options=options))
+
         groups = set(Store.options().groups.keys())
         if kwargs and set(kwargs) <= groups:
             if not all(isinstance(v, dict) for v in kwargs.values()):
