@@ -1,4 +1,5 @@
 import os, sys, warnings, operator
+import time
 import types
 import numbers
 import inspect
@@ -10,6 +11,7 @@ from collections import defaultdict
 from functools import partial
 from contextlib import contextmanager
 
+from threading import Thread, Event
 import numpy as np
 import param
 
@@ -20,11 +22,16 @@ try:
 except:
     from collections import OrderedDict
 
+try:
+   import __builtin__ as builtins # noqa (compatibility)
+except:
+   import builtins as builtins   # noqa (compatibility)
+
 datetime_types = (np.datetime64, dt.datetime)
 
 try:
     import pandas as pd # noqa (optional import)
-    datetime_types = datetime_types + (pd.tslib.Timestamp,)
+    datetime_types = datetime_types + (pd.Timestamp,)
 except ImportError:
     pd = None
 
@@ -35,6 +42,28 @@ except ImportError:
 
 
 
+class Config(param.ParameterizedFunction):
+    """
+    Set of boolean configuration values to change HoloViews' global
+    behavior. Typically used to control warnings relating to
+    deprecations or set global parameter such as style 'themes'.
+    """
+
+    style_17 = param.Boolean(default=False, doc="""
+       Switch to the default style options used up to (and including)
+       the HoloViews 1.7 release.""")
+
+    warn_options_call = param.Boolean(default=False, doc="""
+       Whether to warn when the deprecated __call__ options syntax is
+       used (the opts method should now be used instead). It is
+       recommended that users switch this on to update any uses of
+       __call__ as it will be deprecated in future.""")
+
+    def __call__(self, **params):
+        self.set_param(**params)
+        return self
+
+config = Config()
 
 class HashableJSON(json.JSONEncoder):
     """
@@ -78,6 +107,73 @@ class HashableJSON(json.JSONEncoder):
             return id(obj)
 
 
+class periodic(Thread):
+    """
+    Run a callback count times with a given period without blocking.
+
+    If count is None, will run till timeout (which may be forever if None).
+    """
+
+    def __init__(self, period, count, callback, timeout=None, block=False):
+
+        if isinstance(count, int):
+            if count < 0: raise ValueError('Count value must be positive')
+        elif not type(count) is type(None):
+            raise ValueError('Count value must be a positive integer or None')
+
+        if block is False and count is None and timeout is None:
+            raise ValueError('When using a non-blocking thread, please specify '
+                             'either a count or a timeout')
+
+        super(periodic, self).__init__()
+        self.period = period
+        self.callback = callback
+        self.count = count
+        self.counter = 0
+        self.block = block
+        self.timeout = timeout
+        self._completed = Event()
+        self._start_time = None
+
+    @property
+    def completed(self):
+        return self._completed.is_set()
+
+    def start(self):
+        self._start_time = time.time()
+        if self.block is False:
+            super(periodic,self).start()
+        else:
+            self.run()
+
+    def stop(self):
+        self.timeout = None
+        self._completed.set()
+
+    def __repr__(self):
+        return 'periodic(%s, %s, %s)' % (self.period,
+                                         self.count,
+                                         callable_name(self.callback))
+    def __str__(self):
+        return repr(self)
+
+    def run(self):
+        while not self.completed:
+            if self.block:
+                time.sleep(self.period)
+            else:
+                self._completed.wait(self.period)
+            self.counter += 1
+            self.callback(self.counter)
+
+            if self.timeout is not None:
+                dt = (time.time() - self._start_time)
+                if dt > self.timeout:
+                    self.stop()
+            if self.counter == self.count:
+                self.stop()
+
+
 
 def deephash(obj):
     """
@@ -136,7 +232,7 @@ def argspec(callable_obj):
 
 
 
-def validate_dynamic_argspec(argspec, kdims, streams):
+def validate_dynamic_argspec(callback, kdims, streams):
     """
     Utility used by DynamicMap to ensure the supplied callback has an
     appropriate signature.
@@ -154,7 +250,8 @@ def validate_dynamic_argspec(argspec, kdims, streams):
     If the callback doesn't use **kwargs, the accepted keywords are
     validated against the stream parameter names.
     """
-
+    argspec = callback.argspec
+    name = callback.name
     kdims = [kdim.name for kdim in kdims]
     stream_params = stream_parameters(streams)
     defaults = argspec.defaults if argspec.defaults else []
@@ -166,36 +263,40 @@ def validate_dynamic_argspec(argspec, kdims, streams):
     if argspec.keywords is None:
         unassigned_streams = set(stream_params) - set(argspec.args)
         if unassigned_streams:
-            raise KeyError('Callable missing keywords to accept %s stream parameters'
-                           % ', '.join(unassigned_streams))
+            unassigned = ''.join(unassigned_streams)
+            raise KeyError('Callable {name!r} missing keywords to '
+                           'accept {unassigned} stream parameters'.format(name=name,
+                                                                    unassigned=unassigned))
 
 
     if len(posargs) > len(kdims) + len(stream_params):
-        raise KeyError('Callable accepts more positional arguments than '
-                       'there are kdims and stream parameters')
+        raise KeyError('Callable {name!r} accepts more positional arguments than '
+                       'there are kdims and stream parameters'.format(name=name))
     if kdims == []:                  # Can be no posargs, stream kwargs already validated
         return []
     if set(kdims) == set(posargs):   # Posargs match exactly, can all be passed as kwargs
         return kdims
     elif len(posargs) == len(kdims): # Posargs match kdims length, supplying names
         if argspec.args[:len(kdims)] != posargs:
-            raise KeyError('Unmatched positional kdim arguments only '
-                           'allowed at the start of the signature')
+            raise KeyError('Unmatched positional kdim arguments only allowed at '
+                           'the start of the signature of {name!r}'.format(name=name))
 
         return posargs
     elif argspec.varargs:            # Posargs missing, passed to Callable directly
         return None
     elif set(posargs) - set(kdims):
-        raise KeyError('Callable accepts more positional arguments {posargs} '
-                       'than there are key dimensions {kdims}'.format(posargs=posargs,
+        raise KeyError('Callable {name!r} accepts more positional arguments {posargs} '
+                       'than there are key dimensions {kdims}'.format(name=name,
+                                                                      posargs=posargs,
                                                                       kdims=kdims))
     elif set(kdims).issubset(set(kwargs)): # Key dims can be supplied by keyword
         return kdims
     elif set(kdims).issubset(set(posargs+kwargs)):
         return kdims
     else:
-        raise KeyError('Callback signature over {names} does not accommodate '
-                       'required kdims {kdims}'.format(names=list(set(posargs+kwargs)),
+        raise KeyError('Callback {name!r} signature over {names} does not accommodate '
+                       'required kdims {kdims}'.format(name=name,
+                                                       names=list(set(posargs+kwargs)),
                                                        kdims=kdims))
 
 
@@ -593,7 +694,7 @@ def max_range(ranges):
         with warnings.catch_warnings():
             warnings.filterwarnings('ignore', r'All-NaN (slice|axis) encountered')
             values = [r for r in ranges for v in r if v is not None]
-            if pd and all(isinstance(v, pd.tslib.Timestamp) for r in values for v in r):
+            if pd and all(isinstance(v, pd.Timestamp) for r in values for v in r):
                 values = [(v1.to_datetime64(), v2.to_datetime64()) for v1, v2 in values]
             arr = np.array(values)
             if arr.dtype.kind in 'OSU':
@@ -703,6 +804,7 @@ def unique_array(arr):
     elif pd:
         return pd.unique(arr)
     else:
+        arr = np.asarray(arr)
         _, uniq_inds = np.unique(arr, return_index=True)
         return arr[np.sort(uniq_inds)]
 
@@ -1374,9 +1476,16 @@ def bound_range(vals, density):
     """
     low, high = vals.min(), vals.max()
     invert = False
-    if vals[0] > vals[1]:
+    if len(vals) > 1 and vals[0] > vals[1]:
         invert = True
     if not density:
-        density = round(1./((high-low)/(len(vals)-1)), sys.float_info.dig)
+        with warnings.catch_warnings():
+            warnings.filterwarnings('ignore', r'invalid value encountered in double_scalars')
+            full_precision_density = 1./((high-low)/(len(vals)-1))
+            density = round(full_precision_density, sys.float_info.dig)
+        if density == 0:
+            density = full_precision_density
+    if density == 0:
+        raise ValueError('Could not determine Image density, ensure it has a non-zero range.')
     halfd = 0.5/density
     return low-halfd, high+halfd, density, invert
