@@ -3,38 +3,21 @@ from unittest import SkipTest
 
 import param
 from IPython import version_info
-import numpy as np
 import holoviews
 from param import ipython as param_ext
 from IPython.display import HTML
 
 from ..core.tree import AttrTree
-from ..core.options import Store, Cycle, Palette
+from ..core.options import Store
 from ..element.comparison import ComparisonTestCase
 from ..interface.collector import Collector
+from ..util import extension
 from ..plotting.renderer import Renderer
-from .magics import load_magics, list_formats, list_backends
+from .magics import load_magics
 from .display_hooks import display  # noqa (API import)
-from .display_hooks import set_display_hooks, OutputMagic
+from .display_hooks import set_display_hooks
 from .widgets import RunProgress
 
-try:
-    if version_info[0] >= 4:
-        import nbformat # noqa (ensures availability)
-    else:
-        from IPython import nbformat # noqa (ensures availability)
-    from .archive import notebook_archive
-    holoviews.archive = notebook_archive
-except ImportError:
-    pass
-
-try:
-    import pyparsing
-    from .parser import Parser
-    Parser.namespace = {'np': np, 'Cycle': Cycle, 'Palette': Palette}
-except ImportError:
-    pyparsing = None
-
 
 Collector.interval_hook = RunProgress
 AttrTree._disabled_prefixes = ['_repr_','_ipython_canary_method_should_not_exist']
@@ -90,29 +73,10 @@ class IPTestCase(ComparisonTestCase):
         self.ip.run_line_magic(*args, **kwargs)
 
 
-def load_hvjs(logo=False, JS=True, message='HoloViewsJS successfully loaded.'):
+class notebook_extension(extension):
     """
-    Displays javascript and CSS to initialize HoloViews widgets.
-    """
-    import jinja2
-    # Evaluate load_notebook.html template with widgetjs code
-    if JS:
-        widgetjs, widgetcss = Renderer.html_assets(extras=False, backends=[])
-    else:
-        widgetjs, widgetcss = '', ''
-    templateLoader = jinja2.FileSystemLoader(os.path.dirname(os.path.abspath(__file__)))
-    jinjaEnv = jinja2.Environment(loader=templateLoader)
-    template = jinjaEnv.get_template('load_notebook.html')
-    display(HTML(template.render({'widgetjs': widgetjs,
-                                  'widgetcss': widgetcss,
-                                  'logo': logo,
-                                  'message':message})))
-
-
-class notebook_extension(param.ParameterizedFunction):
-    """
-    Parameterized function to initialize notebook resources
-    and register magics.
+    Notebook specific extension to hv.extension that offers options for
+    controlling the notebook environment.
     """
 
     css = param.String(default='', doc="Optional CSS rule set to apply to the notebook.")
@@ -135,62 +99,44 @@ class notebook_extension(param.ParameterizedFunction):
         using the matplotlib backend) may be used. This may be useful to
         export figures to other formats such as PDF with nbconvert. """)
 
-    ip = param.Parameter(default=None, doc="IPython kernel instance")
+    case_sensitive_completion = param.Boolean(default=False, doc="""
+       Whether to monkey patch IPython to use the correct tab-completion
+       behavior. """)
 
     _loaded = False
 
-    # Mapping between backend name and module name
-    _backends = {'matplotlib': 'mpl',
-                 'bokeh': 'bokeh',
-                 'plotly': 'plotly'}
-
     def __call__(self, *args, **params):
-        # Get requested backends
-        imports = [(arg, self._backends[arg]) for arg in args
-                   if arg in self._backends]
-        for p, val in sorted(params.items()):
-            if p in self._backends:
-                imports.append((p, self._backends[p]))
-        if not imports:
-            args = ['matplotlib']
-            imports = [('matplotlib', 'mpl')]
+        super(notebook_extension, self).__call__(*args, **params)
+        # Abort if IPython not found
+        try:
+            ip = params.pop('ip', None) or get_ipython() # noqa (get_ipython)
+        except:
+            return
 
-        args = list(args)
-        selected_backend = None
-        for backend, imp in imports:
-            try:
-                __import__('holoviews.plotting.%s' % imp)
-                if selected_backend is None:
-                    selected_backend = backend
-            except Exception as e:
-                if backend in args:
-                    args.pop(args.index(backend))
-                if backend in params:
-                    params.pop(backend)
-                if isinstance(e, ImportError):
-                    self.warning("HoloViews %s backend could not be imported, "
-                                 "ensure %s is installed." % (backend, backend))
-                else:
-                    self.warning("Holoviews %s backend could not be imported, "
-                                 "it raised the following exception: %s('%s')" %
-                                 (backend, type(e).__name__, e))
-            finally:
-                if backend == 'matplotlib' and not notebook_extension._loaded:
-                    if 'matplotlib' in Store.renderers:
-                        svg_exporter = Store.renderers['matplotlib'].instance(holomap=None,
-                                                                              fig='svg')
-                        holoviews.archive.exporters = [svg_exporter] +\
-                                                      holoviews.archive.exporters
-                OutputMagic.allowed['backend'] = list_backends()
-                OutputMagic.allowed['fig'] = list_formats('fig', backend)
-                OutputMagic.allowed['holomap'] = list_formats('holomap', backend)
+        # Notebook archive relies on display hooks being set to work.
+        try:
+            if version_info[0] >= 4:
+                import nbformat # noqa (ensures availability)
+            else:
+                from IPython import nbformat # noqa (ensures availability)
+            from .archive import notebook_archive
+            holoviews.archive = notebook_archive
+        except ImportError:
+            pass
+
+        # Not quite right, should be set when switching backends
+        if 'matplotlib' in Store.renderers and not notebook_extension._loaded:
+            svg_exporter = Store.renderers['matplotlib'].instance(holomap=None,fig='svg')
+            holoviews.archive.exporters = [svg_exporter] + holoviews.archive.exporters
+
+        p = param.ParamOverrides(self, {k:v for k,v in params.items() if k!='config'})
+        if p.case_sensitive_completion:
+            from IPython.core import completer
+            completer.completions_sorting_key = self.completions_sorting_key
 
         resources = self._get_resources(args, params)
 
-        ip = params.pop('ip', None)
-        p = param.ParamOverrides(self, params)
         Store.display_formats = p.display_formats
-
         if 'html' not in p.display_formats and len(p.display_formats) > 1:
             msg = ('Output magic unable to control displayed format '
                    'as IPython notebook uses fixed precedence '
@@ -198,18 +144,12 @@ class notebook_extension(param.ParameterizedFunction):
             display(HTML('<b>Warning</b>: %s' % msg))
 
         if notebook_extension._loaded == False:
-            ip = get_ipython() if ip is None else ip # noqa (get_ipython)
             param_ext.load_ipython_extension(ip, verbose=False)
             load_magics(ip)
-            OutputMagic.initialize([backend for backend, _ in imports])
+            Store.output_settings.initialize(list(Store.renderers.keys()))
             set_display_hooks(ip)
             notebook_extension._loaded = True
 
-        if selected_backend is None:
-            raise ImportError('None of the backends could be imported')
-        else:
-            Store.current_backend = selected_backend
-
         css = ''
         if p.width is not None:
             css += '<style>div.container { width: %s%% }</style>' % p.width
@@ -226,8 +166,22 @@ class notebook_extension(param.ParameterizedFunction):
             Store.renderers[r].load_nb(inline=p.inline)
 
         # Create a message for the logo (if shown)
-        load_hvjs(logo=p.logo, JS=('holoviews' in resources), message='')
+        self.load_hvjs(logo=p.logo, JS=('holoviews' in resources), message='')
 
+    @classmethod
+    def completions_sorting_key(cls, word):
+        "Fixed version of IPyton.completer.completions_sorting_key"
+        prio1, prio2 = 0, 0
+        if word.startswith('__'):  prio1 = 2
+        elif word.startswith('_'): prio1 = 1
+        if word.endswith('='):     prio1 = -1
+        if word.startswith('%%'):
+            if not "%" in word[2:]:
+                word = word[2:];   prio2 = 2
+        elif word.startswith('%'):
+            if not "%" in word[1:]:
+                word = word[1:];   prio2 = 1
+        return prio1, word, prio2
 
 
     def _get_resources(self, args, params):
@@ -259,6 +213,24 @@ class notebook_extension(param.ParameterizedFunction):
             resources = ['holoviews'] + resources
         return resources
 
+    @classmethod
+    def load_hvjs(cls, logo=False, JS=True, message='HoloViewsJS successfully loaded.'):
+        """
+        Displays javascript and CSS to initialize HoloViews widgets.
+        """
+        import jinja2
+        # Evaluate load_notebook.html template with widgetjs code
+        if JS:
+            widgetjs, widgetcss = Renderer.html_assets(extras=False, backends=[])
+        else:
+            widgetjs, widgetcss = '', ''
+        templateLoader = jinja2.FileSystemLoader(os.path.dirname(os.path.abspath(__file__)))
+        jinjaEnv = jinja2.Environment(loader=templateLoader)
+        template = jinjaEnv.get_template('load_notebook.html')
+        display(HTML(template.render({'widgetjs': widgetjs,
+                                      'widgetcss': widgetcss,
+                                      'logo': logo,
+                                      'message':message})))
 
     @param.parameterized.bothmethod
     def tab_completion_docstring(self_or_cls):
