@@ -1,6 +1,7 @@
 from __future__ import absolute_import
 
 from collections import Callable, Iterable
+from distutils.version import LooseVersion
 import warnings
 
 import param
@@ -17,7 +18,7 @@ from datashader.dask import dask_pipeline
 from datashape.dispatch import dispatch
 from datashape import discover as dsdiscover
 
-from ..core import (ElementOperation, Element, Dimension, NdOverlay,
+from ..core import (Operation, Element, Dimension, NdOverlay,
                     CompositeOverlay, Dataset)
 from ..core.data import PandasInterface, DaskInterface
 from ..core.util import get_param_values, basestring
@@ -58,7 +59,7 @@ def dataset_pipeline(dataset, schema, canvas, glyph, summary):
     return agg
 
 
-class aggregate(ElementOperation):
+class aggregate(Operation):
     """
     aggregate implements 2D binning for any valid HoloViews Element
     type using datashader. I.e., this operation turns a HoloViews
@@ -108,6 +109,11 @@ class aggregate(ElementOperation):
     y_sampling = param.Number(default=None, doc="""
         Specifies the smallest allowed sampling interval along the y-axis.""")
 
+    target = param.ClassSelector(class_=Image, doc="""
+        A target Image which defines the desired x_range, y_range,
+        width and height.
+    """)
+
     streams = param.List(default=[PlotSize, RangeXY], doc="""
         List of streams that are applied if dynamic=True, allowing
         for dynamic interaction with the plot.""")
@@ -184,34 +190,124 @@ class aggregate(ElementOperation):
         return x, y, Dataset(df, kdims=kdims, vdims=vdims), glyph
 
 
+    def _get_sampling(self, element, x, y):
+        target = self.p.target
+        if target:
+            x_range, y_range = target.range(x), target.range(y)
+            height, width = target.dimension_values(2, flat=False).shape
+        else:
+            if x is None or y is None:
+                x_range = self.p.x_range or (-0.5, 0.5)
+                y_range = self.p.y_range or (-0.5, 0.5)
+            else:
+                x_range = self.p.x_range or element.range(x)
+                y_range = self.p.y_range or element.range(y)
+            width, height = self.p.width, self.p.height
+        (xstart, xend), (ystart, yend) = x_range, y_range
+
+        # Compute highest allowed sampling density
+        xspan = xend - xstart
+        yspan = yend - ystart
+        if self.p.x_sampling:
+            width = int(min([(xspan/self.p.x_sampling), width]))
+        if self.p.y_sampling:
+            height = int(min([(yspan/self.p.y_sampling), height]))
+        xunit, yunit = float(xspan)/width, float(yspan)/height
+        xs, ys = (np.linspace(xstart+xunit/2., xend-xunit/2., width),
+                  np.linspace(ystart+yunit/2., yend-yunit/2., height))
+        return (x_range, y_range), (xs, ys), (width, height)
+
+
+    def _aggregate_ndoverlay(self, element, agg_fn):
+        """
+        Optimized aggregation for NdOverlay objects by aggregating each
+        Element in an NdOverlay individually avoiding having to concatenate
+        items in the NdOverlay. Works by summing sum and count aggregates and
+        applying appropriate masking for NaN values. Mean aggregation
+        is also supported by dividing sum and count aggregates. count_cat
+        aggregates are grouped by the categorical dimension and a separate
+        aggregate for each category is generated.
+        """
+        # Compute overall bounds
+        x, y = element.last.dimensions()[0:2]
+        (x_range, y_range), (xs, ys), (width, height) = self._get_sampling(element, x, y)
+        agg_params = dict({k: v for k, v in self.p.items() if k in aggregate.params()},
+                          x_range=x_range, y_range=y_range)
+
+        # Optimize categorical counts by aggregating them individually
+        if isinstance(agg_fn, ds.count_cat):
+            agg_params.update(dict(dynamic=False, aggregator=ds.count()))
+            agg_fn1 = aggregate.instance(**agg_params)
+            if element.ndims == 1:
+                grouped = element
+            else:
+                grouped = element.groupby([agg_fn.column], container_type=NdOverlay,
+                                          group_type=NdOverlay)
+            return grouped.clone({k: agg_fn1(v) for k, v in grouped.items()})
+
+        # Create aggregate instance for sum, count operations, breaking mean
+        # into two aggregates
+        column = agg_fn.column or 'Count'
+        if isinstance(agg_fn, ds.mean):
+            agg_fn1 = aggregate.instance(**dict(agg_params, aggregator=ds.sum(column)))
+            agg_fn2 = aggregate.instance(**dict(agg_params, aggregator=ds.count()))
+        else:
+            agg_fn1 = aggregate.instance(**agg_params)
+            agg_fn2 = None
+        is_sum = isinstance(agg_fn1.aggregator, ds.sum)
+
+        # Accumulate into two aggregates and mask
+        agg, agg2, mask = None, None, None
+        mask = None
+        for v in element:
+            # Compute aggregates and mask
+            new_agg = agg_fn1.process_element(v, None)
+            if is_sum:
+                new_mask = np.isnan(new_agg.data[column].values)
+                new_agg.data = new_agg.data.fillna(0)
+            if agg_fn2:
+                new_agg2 = agg_fn2.process_element(v, None)
+
+            if agg is None:
+                agg = new_agg
+                if is_sum: mask = new_mask
+                if agg_fn2: agg2 = new_agg2
+            else:
+                agg.data += new_agg.data
+                if is_sum: mask &= new_mask
+                if agg_fn2: agg2.data += new_agg2.data
+
+        # Divide sum by count to compute mean
+        if agg2 is not None:
+            agg2.data.rename({'Count': agg_fn.column}, inplace=True)
+            with np.errstate(divide='ignore', invalid='ignore'):
+                agg.data /= agg2.data
+
+        # Fill masked with with NaNs
+        if is_sum:
+            agg.data[column].values[mask] = np.NaN
+        return agg
+
+
     def _process(self, element, key=None):
         agg_fn = self.p.aggregator
         category = agg_fn.column if isinstance(agg_fn, ds.count_cat) else None
+
+        if (isinstance(element, NdOverlay) and
+            ((isinstance(agg_fn, (ds.count, ds.sum, ds.mean)) and agg_fn.column not in element.kdims) or
+             (isinstance(agg_fn, ds.count_cat) and agg_fn.column in element.kdims))):
+            return self._aggregate_ndoverlay(element, agg_fn)
+
         x, y, data, glyph = self.get_agg_data(element, category)
+        (x_range, y_range), (xs, ys), (width, height) = self._get_sampling(element, x, y)
 
         if x is None or y is None:
-            x0, x1 = self.p.x_range or (-0.5, 0.5)
-            y0, y1 = self.p.y_range or (-0.5, 0.5)
-            xc = np.linspace(x0, x1, self.p.width)
-            yc = np.linspace(y0, y1, self.p.height)
-            xarray = xr.DataArray(np.full((self.p.height, self.p.width), np.NaN, dtype=np.float32),
-                                  dims=['y', 'x'], coords={'x': xc, 'y': yc})
+            xarray = xr.DataArray(np.full((height, width), np.NaN, dtype=np.float32),
+                                  dims=['y', 'x'], coords={'x': xs, 'y': ys})
             return self.p.element_type(xarray)
 
-        xstart, xend = self.p.x_range if self.p.x_range else data.range(x)
-        ystart, yend = self.p.y_range if self.p.y_range else data.range(y)
-
-        # Compute highest allowed sampling density
-        width, height = self.p.width, self.p.height
-        if self.p.x_sampling:
-            x_range = xend - xstart
-            width = int(min([(x_range/self.p.x_sampling), width]))
-        if self.p.y_sampling:
-            y_range = yend - ystart
-            height = int(min([(y_range/self.p.y_sampling), height]))
-
         cvs = ds.Canvas(plot_width=width, plot_height=height,
-                        x_range=(xstart, xend), y_range=(ystart, yend))
+                        x_range=x_range, y_range=y_range)
 
         column = agg_fn.column
         if column and isinstance(agg_fn, ds.count_cat):
@@ -225,16 +321,18 @@ class aggregate(ElementOperation):
 
         agg = getattr(cvs, glyph)(data, x, y, self.p.aggregator)
         if agg.ndim == 2:
-            return self.p.element_type(agg, **params)
+            # Replacing x and y coordinates to avoid numerical precision issues
+            return self.p.element_type((xs, ys, agg.data), **params)
         else:
-            return NdOverlay({c: self.p.element_type(agg.sel(**{column: c}),
-                                                     **params)
-                              for c in agg.coords[column].data},
-                             kdims=[data.get_dimension(column)])
+            layers = {}
+            for c in agg.coords[column].data:
+                cagg = agg.sel(**{column: c})
+                layers[c] = self.p.element_type((xs, ys, cagg.data), **params)
+            return NdOverlay(layers, kdims=[data.get_dimension(column)])
 
 
 
-class shade(ElementOperation):
+class shade(Operation):
     """
     shade applies a normalization function followed by colormapping to
     an Image or NdOverlay of Images, returning an RGB Element.
@@ -259,6 +357,11 @@ class shade(ElementOperation):
         and any valid transfer function that accepts data, mask, nbins
         arguments.""")
 
+    clims = param.NumericTuple(default=None, length=2, doc="""
+        Min and max data values to use for colormap interpolation, when
+        wishing to override autoranging.
+        """)
+
     link_inputs = param.Boolean(default=True, doc="""
          By default, the link_inputs parameter is set to True so that
          when applying shade, backends that support linked streams
@@ -273,7 +376,7 @@ class shade(ElementOperation):
         if not isinstance(overlay, NdOverlay):
             raise ValueError('Only NdOverlays can be concatenated')
         xarr = xr.concat([v.data.T for v in overlay.values()],
-                         dim=overlay.kdims[0].name)
+                         pd.Index(overlay.keys(), name=overlay.kdims[0].name))
         params = dict(get_param_values(overlay.last),
                       vdims=overlay.last.vdims,
                       kdims=overlay.kdims+overlay.last.kdims)
@@ -305,7 +408,8 @@ class shade(ElementOperation):
         else:
             bounds = element.bounds
 
-        array = element.data[element.vdims[0].name]
+        vdim = element.vdims[0].name
+        array = element.data[vdim]
         kdims = element.kdims
 
         # Compute shading options depending on whether
@@ -332,6 +436,11 @@ class shade(ElementOperation):
         else:
             shade_opts['cmap'] = self.p.cmap
 
+        if self.p.clims:
+            shade_opts['span'] = self.p.clims
+        elif LooseVersion(ds.__version__) > '0.5.0' and self.p.normalization != 'eq_hist':
+            shade_opts['span'] = element.range(vdim)
+
         with warnings.catch_warnings():
             warnings.filterwarnings('ignore', r'invalid value encountered in true_divide')
             if np.isnan(array.data).all():
@@ -362,7 +471,7 @@ class datashade(aggregate, shade):
 
 
 
-class dynspread(ElementOperation):
+class dynspread(Operation):
     """
     Spreading expands each pixel in an Image based Element a certain
     number of pixels on all sides according to a given shape, merging
