@@ -5,8 +5,9 @@ import param
 import numpy as np
 import bokeh
 import bokeh.plotting
+from bokeh import palettes
 from bokeh.core.properties import value
-from bokeh.models import  HoverTool, Renderer, Range1d, FactorRange
+from bokeh.models import  HoverTool, Renderer, Range1d, DataRange1d, FactorRange
 from bokeh.models.tickers import Ticker, BasicTicker, FixedTicker, LogTicker
 from bokeh.models.widgets import Panel, Tabs
 from bokeh.models.mappers import LinearColorMapper
@@ -101,7 +102,7 @@ class ElementPlot(BokehPlot, GenericElementPlot):
     show_frame = param.Boolean(default=True, doc="""
         Whether or not to show a complete frame around the plot.""")
 
-    show_grid = param.Boolean(default=True, doc="""
+    show_grid = param.Boolean(default=False, doc="""
         Whether to show a Cartesian grid on the plot.""")
 
     show_legend = param.Boolean(default=True, doc="""
@@ -167,6 +168,10 @@ class ElementPlot(BokehPlot, GenericElementPlot):
     _update_handles = ['source', 'glyph', 'glyph_renderer']
     _categorical = False
 
+    # Declares the default types for continuous x- and y-axes
+    _x_range_type = Range1d
+    _y_range_type = Range1d
+
     def __init__(self, element, plot=None, **params):
         self.current_ranges = None
         super(ElementPlot, self).__init__(element, **params)
@@ -220,6 +225,7 @@ class ElementPlot(BokehPlot, GenericElementPlot):
         dims += element.dimensions()
         return list(util.unique_iterator(dims)), {}
 
+
     def _init_tools(self, element, callbacks=[]):
         """
         Processes the list of tools to be supplied to the plot.
@@ -245,9 +251,12 @@ class ElementPlot(BokehPlot, GenericElementPlot):
 
         tools = [t for t in cb_tools + self.default_tools + self.tools
                  if t not in tool_names]
+        hover_tools = [t for t in tools if isinstance(t, HoverTool)]
         if 'hover' in tools:
             hover = HoverTool(tooltips=tooltips, **hover_opts)
             tools[tools.index('hover')] = hover
+        elif any(hover_tools):
+            hover = hover_tools[0]
         if hover:
             self.handles['hover'] = hover
         return tools
@@ -337,17 +346,20 @@ class ElementPlot(BokehPlot, GenericElementPlot):
         categorical_x = any(isinstance(x, util.basestring) for x in (l, r))
         categorical_y = any(isinstance(y, util.basestring) for y in (b, t))
 
+        range_types = (self._x_range_type, self._y_range_type)
+        if self.invert_axes: range_types = range_types[::-1]
+        x_range_type, y_range_type = range_types
         if categorical or categorical_x:
             x_axis_type = 'auto'
             plot_ranges['x_range'] = FactorRange()
         elif 'x_range' not in plot_ranges:
-            plot_ranges['x_range'] = Range1d()
+            plot_ranges['x_range'] = x_range_type()
 
         if categorical or categorical_y:
             y_axis_type = 'auto'
             plot_ranges['y_range'] = FactorRange()
         elif 'y_range' not in plot_ranges:
-            plot_ranges['y_range'] = Range1d()
+            plot_ranges['y_range'] = y_range_type()
 
         return (x_axis_type, y_axis_type), (xlabel, ylabel, zlabel), plot_ranges
 
@@ -378,9 +390,13 @@ class ElementPlot(BokehPlot, GenericElementPlot):
         if self.toolbar:
             tools = self._init_tools(element)
             properties['tools'] = tools
-            properties['toolbar_location'] = self.toolbar
+        properties['toolbar_location'] = self.toolbar
+
+        if bokeh_version < '0.12.6':
+            properties['webgl'] = self.renderer.webgl
+        elif self.renderer.webgl:
+            properties['output_backend'] =  'webgl'
 
-        properties['webgl'] = Store.renderers[self.renderer.backend].webgl
         with warnings.catch_warnings():
             # Bokeh raises warnings about duplicate tools but these
             # are not really an issue
@@ -394,7 +410,7 @@ class ElementPlot(BokehPlot, GenericElementPlot):
         """
         Returns a dictionary of plot properties.
         """
-        size_multiplier = Store.renderers[self.renderer.backend].size/100.
+        size_multiplier = self.renderer.size/100.
         plot_props = dict(plot_height=int(self.height*size_multiplier),
                           plot_width=int(self.width*size_multiplier))
         if bokeh_version < '0.12':
@@ -476,7 +492,15 @@ class ElementPlot(BokehPlot, GenericElementPlot):
                 axis_props['ticker'] = BasicTicker(desired_num_ticks=ticker)
             elif isinstance(ticker, (tuple, list)):
                 if all(isinstance(t, tuple) for t in ticker):
-                    pass
+                    ticks, labels = zip(*ticker)
+                    labels = [l if isinstance(l, util.basestring) else str(l)
+                              for l in labels]
+                    axis_props['ticker'] = FixedTicker(ticks=ticks)
+                    if bokeh_version > '0.12.5':
+                        axis_props['major_label_overrides'] = dict(zip(ticks, labels))
+                    else:
+                        self.warning('Explicit tick labels not supported until'
+                                     'bokeh 0.12.6, please upgrade')
                 else:
                     axis_props['ticker'] = FixedTicker(ticks=ticker)
 
@@ -523,7 +547,7 @@ class ElementPlot(BokehPlot, GenericElementPlot):
         y_range = self.handles['y_range']
 
         l, b, r, t = None, None, None, None
-        if any(isinstance(r, Range1d) for r in [x_range, y_range]):
+        if any(isinstance(r, (Range1d, DataRange1d)) for r in [x_range, y_range]):
             l, b, r, t = self.get_extents(element, ranges)
             if self.invert_axes:
                 l, b, r, t = b, l, t, r
@@ -539,7 +563,7 @@ class ElementPlot(BokehPlot, GenericElementPlot):
 
 
     def _update_range(self, axis_range, low, high, factors, invert, shared):
-        if isinstance(axis_range, Range1d):
+        if isinstance(axis_range, (Range1d, DataRange1d)) and self.apply_ranges:
             if (low == high and low is not None and
                 not isinstance(high, util.datetime_types)):
                 offset = abs(low*0.1 if low else 0.5)
@@ -565,7 +589,7 @@ class ElementPlot(BokehPlot, GenericElementPlot):
         """
         Transforms non-string or integer types in datasource if the
         axis to be plotted on is categorical. Accepts the column data
-        sourcec data, the columns corresponding to the axes and the
+        source data, the columns corresponding to the axes and the
         dimensions for each axis, changing the data inplace.
         """
         if self.invert_axes:
@@ -585,8 +609,8 @@ class ElementPlot(BokehPlot, GenericElementPlot):
         xdim, ydim = element.dimensions()[:2]
         xvals, yvals = [element.dimension_values(i, False)
                         for i in range(2)]
-        coords = ([x if xvals.dtype.kind in 'SU' else xdim.pprint_value(x) for x in xvals],
-                  [y if yvals.dtype.kind in 'SU' else ydim.pprint_value(y) for y in yvals])
+        coords = ([x if xvals.dtype.kind in 'SU' else xdim.pprint_value(x).replace(':', ';') for x in xvals],
+                  [y if yvals.dtype.kind in 'SU' else ydim.pprint_value(y).replace(':', ';') for y in yvals])
         if self.invert_axes: coords = coords[::-1]
         return coords
 
@@ -610,6 +634,9 @@ class ElementPlot(BokehPlot, GenericElementPlot):
         """
         properties = mpl_to_bokeh(properties)
         plot_method = self._plot_methods.get('batched' if self.batched else 'single')
+        if isinstance(plot_method, tuple):
+            # Handle alternative plot method for flipped axes
+            plot_method = plot_method[int(self.invert_axes)]
         renderer = getattr(plot, plot_method)(**dict(properties, **mapping))
         return renderer, renderer.glyph
 
@@ -627,7 +654,7 @@ class ElementPlot(BokehPlot, GenericElementPlot):
         properties['source'] = source
         return properties
 
-    def _update_glyphs(self, renderer, properties, mapping, glyph):
+    def _update_glyph(self, renderer, properties, mapping, glyph):
         allowed_properties = glyph.properties()
         properties = mpl_to_bokeh(properties)
         merged = dict(properties, **mapping)
@@ -676,7 +703,7 @@ class ElementPlot(BokehPlot, GenericElementPlot):
         hover = self.handles.get('hover')
         if hover is None:
             return
-        hover.renderers = [renderer]
+        hover.renderers.append(renderer)
 
         # If datetime column is in the data replace hover formatter
         for k, v in source.data.items():
@@ -689,6 +716,37 @@ class ElementPlot(BokehPlot, GenericElementPlot):
                 hover.tooltips = tooltips
 
 
+    def _init_glyphs(self, plot, element, ranges, source):
+        empty = False
+        style_element = element.last if self.batched else element
+
+        # Get data and initialize data source
+        empty = False
+        if self.batched:
+            current_id = tuple(element.traverse(lambda x: x._plot_id, [Element]))
+            data, mapping = self.get_batched_data(element, ranges, empty)
+        else:
+            data, mapping = self.get_data(element, ranges, empty)
+            current_id = element._plot_id
+        if source is None:
+            source = self._init_datasource(data)
+        self.handles['previous_id'] = current_id
+        self.handles['source'] = source
+
+        properties = self._glyph_properties(plot, style_element, source, ranges)
+        with abbreviated_exception():
+            renderer, glyph = self._init_glyph(plot, mapping, properties)
+        self.handles['glyph'] = glyph
+        if isinstance(renderer, Renderer):
+            self.handles['glyph_renderer'] = renderer
+
+        self._postprocess_hover(renderer, source)
+
+        # Update plot, source and glyph
+        with abbreviated_exception():
+            self._update_glyph(renderer, properties, mapping, glyph)
+
+
     def initialize_plot(self, ranges=None, plot=None, plots=None, source=None):
         """
         Initializes a new plot object with the last available frame.
@@ -717,35 +775,13 @@ class ElementPlot(BokehPlot, GenericElementPlot):
             self.handles['y_range'] = plot.y_range
         self.handles['plot'] = plot
 
-        # Get data and initialize data source
-        empty = False
-        if self.batched:
-            data, mapping = self.get_batched_data(element, ranges, empty)
-        else:
-            data, mapping = self.get_data(element, ranges, empty)
-        if source is None:
-            source = self._init_datasource(data)
-        self.handles['source'] = source
-
-        properties = self._glyph_properties(plot, style_element, source, ranges)
-        with abbreviated_exception():
-            renderer, glyph = self._init_glyph(plot, mapping, properties)
-        self.handles['glyph'] = glyph
-        if isinstance(renderer, Renderer):
-            self.handles['glyph_renderer'] = renderer
-
-        self._postprocess_hover(renderer, source)
-
-        # Update plot, source and glyph
-        with abbreviated_exception():
-            self._update_glyphs(renderer, properties, mapping, glyph)
+        self._init_glyphs(plot, element, ranges, source)
         if not self.overlaid:
             self._update_plot(key, plot, style_element)
             self._update_ranges(style_element, ranges)
 
-        if not self.batched:
-            for cb in self.callbacks:
-                cb.initialize()
+        for cb in self.callbacks:
+            cb.initialize()
 
         if not self.overlaid:
             self._process_legend()
@@ -756,6 +792,36 @@ class ElementPlot(BokehPlot, GenericElementPlot):
         return plot
 
 
+    def _update_glyphs(self, element, ranges):
+        plot = self.handles['plot']
+        glyph = self.handles.get('glyph')
+        source = self.handles['source']
+        empty = False
+        mapping = {}
+
+        # Cache frame object id to skip updating data if unchanged
+        previous_id = self.handles.get('previous_id', None)
+        if self.batched:
+            current_id = tuple(element.traverse(lambda x: x._plot_id, [Element]))
+        else:
+            current_id = element._plot_id
+        self.handles['previous_id'] = current_id
+        self.static_source = (self.dynamic and (current_id == previous_id))
+        if self.batched:
+            data, mapping = self.get_batched_data(element, ranges, empty)
+        else:
+            data, mapping = self.get_data(element, ranges, empty)
+
+        if not self.static_source:
+            self._update_datasource(source, data)
+
+        if glyph:
+            properties = self._glyph_properties(plot, element, source, ranges)
+            renderer = self.handles.get('glyph_renderer')
+            with abbreviated_exception():
+                self._update_glyph(renderer, properties, mapping, glyph)
+
+
     def update_frame(self, key, ranges=None, plot=None, element=None, empty=False):
         """
         Updates an existing plot with data corresponding
@@ -768,7 +834,6 @@ class ElementPlot(BokehPlot, GenericElementPlot):
             self.current_key = key
             self.current_frame = element
 
-        glyph = self.handles.get('glyph', None)
         renderer = self.handles.get('glyph_renderer', None)
         if hasattr(renderer, 'visible'):
             renderer.visible = bool(element)
@@ -789,32 +854,9 @@ class ElementPlot(BokehPlot, GenericElementPlot):
         self.set_param(**self.lookup_options(style_element, 'plot').options)
         ranges = util.match_spec(style_element, ranges)
         self.current_ranges = ranges
+        self._update_glyphs(element, ranges)
 
         plot = self.handles['plot']
-        source = self.handles['source']
-        empty = False
-        mapping = {}
-
-        # Cache frame object id to skip updating data if unchanged
-        previous_id = self.handles.get('previous_id', None)
-        if self.batched:
-            current_id = sum(element.traverse(lambda x: id(x.data), [Element]))
-        else:
-            current_id = id(element.data)
-        self.handles['previous_id'] = current_id
-        self.static_source = self.dynamic and (current_id == previous_id)
-        if self.batched:
-            data, mapping = self.get_batched_data(element, ranges, empty)
-        else:
-            data, mapping = self.get_data(element, ranges, empty)
-        if not self.static_source:
-            self._update_datasource(source, data)
-
-        if glyph:
-            properties = self._glyph_properties(plot, element, source, ranges)
-            renderer = self.handles.get('glyph_renderer')
-            with abbreviated_exception():
-                self._update_glyphs(renderer, properties, mapping, glyph)
         if not self.overlaid:
             self._update_ranges(style_element, ranges)
             self._update_plot(key, plot, style_element)
@@ -898,6 +940,95 @@ class ElementPlot(BokehPlot, GenericElementPlot):
                    for frame in current_frames)
 
 
+class CompositeElementPlot(ElementPlot):
+    """
+    A CompositeElementPlot is an Element plot type that coordinates
+    drawing of multiple glyphs.
+    """
+
+    # Mapping between glyph name and style groups
+    _style_groups = {}
+
+    def _init_glyphs(self, plot, element, ranges, source):
+        # Get data and initialize data source
+        empty = False
+        if self.batched:
+            current_id = tuple(element.traverse(lambda x: x._plot_id, [Element]))
+            data, mapping = self.get_batched_data(element, ranges, empty)
+        else:
+            data, mapping = self.get_data(element, ranges, empty)
+            current_id = element._plot_id
+
+        self.handles['previous_id'] = current_id
+        for key in dict(mapping, **data):
+            source = self._init_datasource(data.get(key, {}))
+            self.handles[key+'_source'] = source
+            properties = self._glyph_properties(plot, element, source, ranges)
+            properties = self._process_properties(key, properties)
+            with abbreviated_exception():
+                renderer, glyph = self._init_glyph(plot, mapping.get(key, {}), properties, key)
+            self.handles[key+'_glyph'] = glyph
+            if isinstance(renderer, Renderer):
+                self.handles[key+'glyph_renderer'] = renderer
+
+            self._postprocess_hover(renderer, source)
+
+            # Update plot, source and glyph
+            with abbreviated_exception():
+                self._update_glyph(renderer, properties, mapping.get(key, {}), glyph)
+
+
+    def _process_properties(self, key, properties):
+        style_group = self._style_groups[key.split('_')[0]]
+        group_props = {}
+        for k, v in properties.items():
+            if k in self.style_opts:
+                if k.split('_')[0] == style_group:
+                    k = '_'.join(k.split('_')[1:])
+                else:
+                    continue
+            group_props[k] = v
+        return group_props
+
+
+    def _update_glyphs(self, element, ranges):
+        plot = self.handles['plot']
+        empty = False
+
+        # Cache frame object id to skip updating data if unchanged
+        previous_id = self.handles.get('previous_id', None)
+        if self.batched:
+            current_id = tuple(element.traverse(lambda x: x._plot_id, [Element]))
+        else:
+            current_id = element._plot_id
+        self.handles['previous_id'] = current_id
+        self.static_source = (self.dynamic and (current_id == previous_id))
+        data, mapping = self.get_data(element, ranges, empty)
+
+        for key, gdata in data.items():
+            source = self.handles[key+'_source']
+            glyph = self.handles[key+'_glyph']
+            if not self.static_source:
+                self._update_datasource(source, gdata)
+
+            if glyph:
+                properties = self._glyph_properties(plot, element, source, ranges)
+                properties = self._process_properties(key, properties)
+                renderer = self.handles.get(key+'_glyph_renderer')
+                with abbreviated_exception():
+                    self._update_glyph(renderer, properties, mapping[key], glyph)
+
+
+    def _init_glyph(self, plot, mapping, properties, key):
+        """
+        Returns a Bokeh glyph object.
+        """
+        properties = mpl_to_bokeh(properties)
+        plot_method = key.split('_')[0]
+        renderer = getattr(plot, plot_method)(**dict(properties, **mapping))
+        return renderer, renderer.glyph
+
+
 
 class ColorbarPlot(ElementPlot):
     """
@@ -982,7 +1113,7 @@ class ColorbarPlot(ElementPlot):
         self.handles['colorbar'] = color_bar
 
 
-    def _get_colormapper(self, dim, element, ranges, style, factors=None):
+    def _get_colormapper(self, dim, element, ranges, style, factors=None, colors=None):
         # The initial colormapper instance is cached the first time
         # and then only updated
         if dim is None:
@@ -1000,9 +1131,26 @@ class ColorbarPlot(ElementPlot):
 
         ncolors = None if factors is None else len(factors)
         low, high = ranges.get(dim.name, element.range(dim.name))
-        palette = mplcmap_to_palette(style.pop('cmap', 'viridis'), ncolors)
-        colors = {k: rgba_tuple(v) for k, v in self.clipping_colors.items()}
-        colormapper, opts = self._get_cmapper_opts(low, high, factors, colors)
+        if colors:
+            palette = colors
+        else:
+            cmap = style.pop('cmap', 'viridis')
+            if isinstance(cmap, list):
+                palette = cmap
+            else:
+                try:
+                    # Process as matplotlib colormap
+                    palette = mplcmap_to_palette(cmap, ncolors)
+                except ValueError:
+                    # Process as bokeh palette
+                    palette = getattr(palettes, cmap, None)
+                    if isinstance(palette, dict):
+                        if ncolors in palette:
+                            palette = palette[ncolors]
+                        else:
+                            palette = sorted(palette.items())[-1][1]
+        nan_colors = {k: rgba_tuple(v) for k, v in self.clipping_colors.items()}
+        colormapper, opts = self._get_cmapper_opts(low, high, factors, nan_colors)
 
         if 'color_mapper' in self.handles:
             cmapper = self.handles['color_mapper']
@@ -1015,16 +1163,15 @@ class ColorbarPlot(ElementPlot):
         return cmapper
 
 
-    def _get_color_data(self, element, ranges, style, name='color'):
+    def _get_color_data(self, element, ranges, style, name='color', factors=None, colors=None):
         data, mapping = {}, {}
         cdim = element.get_dimension(self.color_index)
         if cdim:
             cdata = element.dimension_values(cdim)
-            factors = None
-            if isinstance(cdata, list) or cdata.dtype.kind in 'OSU':
+            if factors is None and (isinstance(cdata, list) or cdata.dtype.kind in 'OSU'):
                 factors = list(np.unique(cdata))
             mapper = self._get_colormapper(cdim, element, ranges, style,
-                                           factors)
+                                           factors, colors)
             data[cdim.name] = cdata
             if factors is not None:
                 mapping['legend'] = {'field': cdim.name}
@@ -1090,7 +1237,12 @@ class LegendPlot(ElementPlot):
         if not plot.legend:
             return
         legend = plot.legend[0]
-        if (not self.overlaid and len(legend.items) == 1) or not self.show_legend:
+        cmapper = self.handles.get('color_mapper')
+        if cmapper:
+            categorical = isinstance(cmapper, CategoricalColorMapper)
+        else:
+            categorical = False
+        if (not categorical and  not self.overlaid and len(legend.items) == 1) or not self.show_legend:
             legend.items[:] = []
         else:
             plot.legend.orientation = 'horizontal' if self.legend_cols else 'vertical'
@@ -1116,7 +1268,7 @@ class OverlayPlot(GenericOverlayPlot, LegendPlot):
 
     _propagate_options = ['width', 'height', 'xaxis', 'yaxis', 'labelled',
                           'bgcolor', 'fontsize', 'invert_axes', 'show_frame',
-                          'show_grid', 'logx', 'logy', 'xticks',
+                          'show_grid', 'logx', 'logy', 'xticks', 'toolbar',
                           'yticks', 'xrotation', 'yrotation', 'lod',
                           'border', 'invert_xaxis', 'invert_yaxis']
 
@@ -1189,7 +1341,7 @@ class OverlayPlot(GenericOverlayPlot, LegendPlot):
                         tool_type = type(tool)
                     if isinstance(tool, HoverTool):
                         if tuple(tool.tooltips) in hover_tools:
-                            hover_tools[tuple(tool.tooltips)].renderers += tool.renderers
+                            continue
                         else:
                             hover_tools[tuple(tool.tooltips)] = tool
                     elif tool_type in tool_types:
@@ -1205,14 +1357,18 @@ class OverlayPlot(GenericOverlayPlot, LegendPlot):
         """
         Merges tools on the overlay with those on the subplots.
         """
-        if 'hover' in subplot.handles and 'hover_tools' in self.handles:
-            hover = subplot.handles['hover']
-            tool = self.handles['hover_tools'].get(tuple(hover.tooltips))
-            if tool:
-                tool.renderers += hover.renderers
-        elif self.batched and 'hover' in subplot.handles:
+        if self.batched and 'hover' in subplot.handles:
             self.handles['hover'] = subplot.handles['hover']
-
+        elif 'hover' in subplot.handles and 'hover_tools' in self.handles:
+            hover = subplot.handles['hover']
+            # Datetime formatter may have been applied, remove _dt_strings
+            # to match on the hover tooltips, then merge tool renderers
+            tooltips = [(name, spec.replace('_dt_strings', ''))
+                        for name, spec in hover.tooltips]
+            tool = self.handles['hover_tools'].get(tuple(tooltips))
+            if tool:
+                renderers = tool.renderers+hover.renderers
+                tool.renderers = list(util.unique_iterator(renderers))
 
     def _get_factors(self, overlay):
         xfactors, yfactors = [], []
@@ -1256,7 +1412,9 @@ class OverlayPlot(GenericOverlayPlot, LegendPlot):
             if self.batched:
                 self.handles['plot'] = child
             if self.tabs:
-                title = get_tab_title(key, frame, self.hmap.last)
+                title = subplot._format_title(key, dimensions=False)
+                if not title:
+                    title = get_tab_title(key, frame, self.hmap.last)
                 panels.append(Panel(child=child, title=title))
             self._merge_tools(subplot)
 
