@@ -1,4 +1,4 @@
-import itertools, inspect, re
+import itertools, inspect, re, time
 from distutils.version import LooseVersion
 from collections import defaultdict
 
@@ -17,30 +17,37 @@ from bokeh.core.enums import Palette
 from bokeh.core.json_encoder import serialize_json # noqa (API import)
 from bokeh.core.properties import value
 from bokeh.document import Document
-from bokeh.models import (GlyphRenderer, Model, HasProps, Column, Row,
-                          ToolbarBox, FactorRange, Range1d)
-from bokeh.models.widgets import DataTable, Tabs
+from bokeh.layouts import WidgetBox, Row, Column
+from bokeh.models import Model, HasProps, ToolbarBox, FactorRange, Range1d, Plot
+from bokeh.models.widgets import DataTable, Tabs, Div
 from bokeh.plotting import Figure
-if bokeh_version >= '0.12':
-    from bokeh.layouts import WidgetBox
+
+try:
+    if bokeh_version > '0.12.5':
+        from bkcharts import Chart
+    else:
+        from bokeh.charts import Chart
+except:
+    Chart = type(None) # Create stub for isinstance check
 
 from ...core.options import abbreviated_exception
 from ...core.overlay import Overlay
-from ...core.util import basestring, unique_array
+from ...core.util import basestring, unique_array, callable_name
+from ...core.spaces import get_nested_dmaps, DynamicMap
 
 from ..util import dim_axis_label, rgb2hex
 
 # Conversion between matplotlib and bokeh markers
 markers = {'s': {'marker': 'square'},
            'd': {'marker': 'diamond'},
-           '^': {'marker': 'triangle', 'orientation': 0},
-           '>': {'marker': 'triangle', 'orientation': np.pi/2},
-           'v': {'marker': 'triangle', 'orientation': np.pi},
-           '<': {'marker': 'triangle', 'orientation': -np.pi/2},
-           '1': {'marker': 'triangle', 'orientation': 0},
-           '2': {'marker': 'triangle', 'orientation': np.pi/2},
-           '3': {'marker': 'triangle', 'orientation': np.pi},
-           '4': {'marker': 'triangle', 'orientation': -np.pi/2}}
+           '^': {'marker': 'triangle', 'angle': 0},
+           '>': {'marker': 'triangle', 'angle': -np.pi/2},
+           'v': {'marker': 'triangle', 'angle': np.pi},
+           '<': {'marker': 'triangle', 'angle': np.pi/2},
+           '1': {'marker': 'triangle', 'angle': 0},
+           '2': {'marker': 'triangle', 'angle': -np.pi/2},
+           '3': {'marker': 'triangle', 'angle': np.pi},
+           '4': {'marker': 'triangle', 'angle': np.pi/2}}
 
 # List of models that do not update correctly and must be ignored
 # Should only include models that have no direct effect on the display
@@ -55,10 +62,9 @@ IGNORED_MODELS = ['LinearAxis', 'LogAxis', 'DatetimeAxis', 'DatetimeTickFormatte
 IGNORED_ATTRIBUTES = ['data', 'palette', 'image', 'x', 'y', 'factors']
 
 # Model priority order to ensure some types are updated before others
-MODEL_PRIORITY = ['Range1d', 'Title', 'Image', 'LinearColorMapper',
-                  'Plot', 'Range1d', 'FactorRange', 'CategoricalAxis',
-                  'LinearAxis', 'ColumnDataSource']
-
+MODEL_PRIORITY = ['DataRange1d', 'Range1d', 'Title', 'Image', 'LinearColorMapper',
+                  'Plot', 'FactorRange', 'CategoricalAxis', 'LinearAxis',
+                  'ColumnDataSource']
 
 
 def rgba_tuple(rgba):
@@ -146,6 +152,31 @@ def layout_padding(plots, renderer):
     return expanded_plots
 
 
+def compute_plot_size(plot):
+    """
+    Computes the size of bokeh models that make up a layout such as
+    figures, rows, columns, widgetboxes and Plot.
+    """
+    if isinstance(plot, Div):
+        # Cannot compute size for Div
+        return 0, 0
+    elif isinstance(plot, (Row, Column, ToolbarBox, WidgetBox, Tabs)):
+        if not plot.children: return 0, 0
+        if isinstance(plot, Row) or (isinstance(plot, ToolbarBox) and plot.toolbar_location not in ['right', 'left']):
+            w_agg, h_agg = (np.sum, np.max)
+        elif isinstance(plot, Tabs):
+            w_agg, h_agg = (np.max, np.max)
+        else:
+            w_agg, h_agg = (np.max, np.sum)
+        widths, heights = zip(*[compute_plot_size(child) for child in plot.children])
+        width, height = w_agg(widths), h_agg(heights)
+    elif isinstance(plot, (Figure, Chart)):
+        width, height = plot.plot_width, plot.plot_height
+    elif isinstance(plot, (Plot, DataTable)):
+        width, height = plot.width, plot.height
+    return width, height
+
+
 def empty_plot(width, height):
     """
     Creates an empty and invisible plot of the specified size.
@@ -425,38 +456,6 @@ def hsv_to_rgb(hsv):
     return rgb.reshape(shape+(3,))
 
 
-def update_plot(old, new):
-    """
-    Updates an existing plot or figure with a new plot,
-    useful for bokeh charts and mpl conversions, which do
-    not allow updating an existing plot easily.
-
-    ALERT: Should be replaced once bokeh supports it directly
-    """
-    old_renderers = old.select(type=GlyphRenderer)
-    new_renderers = new.select(type=GlyphRenderer)
-
-    old.x_range.update(**new.x_range.properties_with_values())
-    old.y_range.update(**new.y_range.properties_with_values())
-    updated = []
-    for new_r in new_renderers:
-        for old_r in old_renderers:
-            if type(old_r.glyph) == type(new_r.glyph):
-                old_renderers.pop(old_renderers.index(old_r))
-                new_props = new_r.properties_with_values()
-                source = new_props.pop('data_source')
-                old_r.glyph.update(**new_r.glyph.properties_with_values())
-                old_r.update(**new_props)
-                old_r.data_source.data.update(source.data)
-                updated.append(old_r)
-                break
-
-    for old_r in old_renderers:
-        if old_r not in updated:
-            emptied = {k: [] for k in old_r.data_source.data}
-            old_r.data_source.data.update(emptied)
-
-
 def pad_width(model, table_padding=0.85, tabs_padding=1.2):
     """
     Computes the width of a model and sets up appropriate padding
@@ -656,3 +655,90 @@ def update_shared_sources(f):
             source.data.update(patch)
         return ret
     return wrapper
+
+
+def categorize_array(array, dim):
+    """
+    Uses a Dimension instance to convert an array of values to categorical
+    (i.e. string) values and applies escaping for colons, which bokeh
+    treats as a categorical suffix.
+    """
+    return np.array([dim.pprint_value(x).replace(':', ';') for x in array])
+
+
+class periodic(object):
+    """
+    Mocks the API of periodic Thread in hv.core.util, allowing a smooth
+    API transition on bokeh server.
+    """
+
+    def __init__(self, document):
+        self.document = document
+        self.callback = None
+        self.period = None
+        self.count = None
+        self.counter = None
+        self._start_time = None
+        self.timeout = None
+
+    @property
+    def completed(self):
+        return self.counter is None
+
+    def start(self):
+        self._start_time = time.time()
+        if self.document is None:
+            raise RuntimeError('periodic was registered to be run on bokeh'
+                               'server but no document was found.')
+        self.document.add_periodic_callback(self._periodic_callback, self.period)
+
+    def __call__(self, period, count, callback, timeout=None, block=False):
+        if isinstance(count, int):
+            if count < 0: raise ValueError('Count value must be positive')
+        elif not type(count) is type(None):
+            raise ValueError('Count value must be a positive integer or None')
+
+        self.callback = callback
+        self.period = period*1000.
+        self.timeout = timeout
+        self.count = count
+        self.counter = 0
+        return self
+
+    def _periodic_callback(self):
+        self.callback(self.counter)
+        self.counter += 1
+
+        if self.timeout is not None:
+            dt = (time.time() - self._start_time)
+            if dt > self.timeout:
+                self.stop()
+        if self.counter == self.count:
+            self.stop()
+
+    def stop(self):
+        self.counter = None
+        self.timeout = None
+        try:
+            self.document.remove_periodic_callback(self._periodic_callback)
+        except ValueError: # Already stopped
+            pass
+
+    def __repr__(self):
+        return 'periodic(%s, %s, %s)' % (self.period,
+                                         self.count,
+                                         callable_name(self.callback))
+    def __str__(self):
+        return repr(self)
+
+
+
+
+def attach_periodic(plot):
+    """
+    Attaches plot refresh to all streams on the object.
+    """
+    def append_refresh(dmap):
+        for dmap in get_nested_dmaps(dmap):
+            dmap.periodic._periodic_util = periodic(plot.document)
+    return plot.hmap.traverse(append_refresh, [DynamicMap])
