@@ -25,12 +25,13 @@ def _rc_context(rcparams):
     """
     Context manager that temporarily overrides the pyplot rcParams.
     """
-    old_rcparams = plt.rcParams.copy()
-    plt.rcParams.update(rcparams)
+    old_rcparams = mpl.rcParams.copy()
+    mpl.rcParams.update(rcparams)
     try:
         yield
     finally:
-        plt.rcParams = old_rcparams
+        mpl.rcParams.clear()
+        mpl.rcParams.update(old_rcparams)
 
 def mpl_rc_context(f):
     """
@@ -114,7 +115,7 @@ class MPLPlot(DimensionedPlot):
         May also supply a custom projection that is either a matplotlib
         projection type or implements the `_as_mpl_axes` method.""")
 
-    show_frame = param.Boolean(default=True, doc="""
+    show_frame = param.Boolean(default=False, doc="""
         Whether or not to show a complete frame around the plot.""")
 
     _close_figures = True
@@ -138,8 +139,7 @@ class MPLPlot(DimensionedPlot):
         elif not self.renderer.notebook_context:
             plt.ioff()
 
-        with mpl.rc_context(rc=self.fig_rcparams):
-            fig, axis = self._init_axis(fig, axis)
+        fig, axis = self._init_axis(fig, axis)
 
         self.handles['fig'] = fig
         self.handles['axis'] = axis
@@ -151,6 +151,7 @@ class MPLPlot(DimensionedPlot):
         self.handles['bbox_extra_artists'] = []
 
 
+    @mpl_rc_context
     def _init_axis(self, fig, axis):
         """
         Return an axis which may need to be initialized from
@@ -286,9 +287,6 @@ class GridPlot(CompositePlot):
         If enabled the x-axes of the GridSpace will be drawn from the
         objects inside the Grid rather than the GridSpace dimensions.""")
 
-    show_frame = param.Boolean(default=False, doc="""
-        Whether to draw a frame around the Grid.""")
-
     show_legend = param.Boolean(default=False, doc="""
         Legends add to much clutter in a grid and are disabled by default.""")
 
@@ -919,9 +917,11 @@ class LayoutPlot(GenericLayoutPlot, CompositePlot):
             # axis objects, handling any Empty objects.
             obj = layouts[(r, c)]
             empty = isinstance(obj.main, Empty)
-            if empty:
+            if view.main is None:
+                continue
+            elif empty:
                 obj = AdjointLayout([])
-            elif view is None or not view.traverse(lambda x: x, [Element]):
+            elif not view.traverse(lambda x: x, [Element]):
                 self.warning('%s is empty, skipping subplot.' % obj.main)
                 continue
             elif self.transpose:
@@ -1110,8 +1110,6 @@ class AdjoinedPlot(DimensionedPlot):
     border_size = param.Number(default=0.25, doc="""
         The size of the border expressed as a fraction of the main plot.""")
 
-    show_frame = param.Boolean(default=False)
-
     show_title = param.Boolean(default=False, doc="""
         Titles should be disabled on all SidePlots to avoid clutter.""")
 
