@@ -11,7 +11,7 @@ from contextlib import contextmanager
 import param
 from ..core.io import Exporter
 from ..core.options import Store, StoreOptions, SkipRendering
-from ..core.util import find_file, unicode, unbound_dimensions
+from ..core.util import find_file, unicode, unbound_dimensions, basestring
 from .. import Layout, HoloMap, AdjointLayout
 from .widgets import NdWidget, ScrubberWidget, SelectionWidget
 
@@ -459,14 +459,16 @@ class Renderer(Exporter):
         supplied format or to the appropriate default.
         """
         if info or key:
-            raise Exception('MPLRenderer does not support saving metadata to file.')
+            raise Exception('Renderer does not support saving metadata to file.')
 
         with StoreOptions.options(obj, options, **kwargs):
             plot = self_or_cls.get_plot(obj)
 
         if (fmt in list(self_or_cls.widgets.keys())+['auto']) and len(plot) > 1:
             with StoreOptions.options(obj, options, **kwargs):
-                self_or_cls.export_widgets(plot, basename+'.html', fmt)
+                if isinstance(basename, basestring):
+                    basename = basename+'.html'
+                self_or_cls.export_widgets(plot, basename, fmt)
             return
 
         with StoreOptions.options(obj, options, **kwargs):
@@ -474,6 +476,9 @@ class Renderer(Exporter):
         if rendered is None: return
         (data, info) = rendered
         encoded = self_or_cls.encode(rendered)
+        prefix = self_or_cls._save_prefix(info['file-ext'])
+        if prefix:
+            encoded = prefix + encoded
         if isinstance(basename, BytesIO):
             basename.write(encoded)
             basename.seek(0)
@@ -482,6 +487,11 @@ class Renderer(Exporter):
             with open(filename, 'wb') as f:
                 f.write(encoded)
 
+    @bothmethod
+    def _save_prefix(self_or_cls, ext):
+        "Hook to prefix content for instance JS when saving HTML"
+        return
+
 
     @bothmethod
     def get_size(self_or_cls, plot):
