@@ -293,7 +293,8 @@ class Stream(param.Parameterized):
     @property
     def contents(self):
         filtered = {k:v for k,v in self.get_param_values() if k!= 'name' }
-        return {self._rename.get(k,k):v for (k,v) in filtered.items()}
+        return {self._rename.get(k,k):v for (k,v) in filtered.items()
+                if (self._rename.get(k,True) is not None)}
 
 
     def _set_stream_parameters(self, **kwargs):
@@ -412,6 +413,22 @@ class PointerXY(LinkedStream):
            Pointer position along the y-axis in data coordinates""")
 
 
+class Draw(PointerXY):
+    """
+    A series of updating x/y-positions when drawing, together with the
+    current stroke count
+    """
+
+    stroke_count = param.Integer(default=0, constant=True, doc="""
+       The current drawing stroke count. Increments every time a new
+       stroke is started.""")
+
+class SingleTap(PointerXY):
+    """
+    The x/y-position of a single tap or click in data coordinates.
+    """
+
+
 class Tap(PointerXY):
     """
     The x/y-position of a tap or click in data coordinates.
@@ -493,7 +510,29 @@ class Bounds(LinkedStream):
 
     bounds = param.NumericTuple(default=None, constant=True, length=4,
                                 allow_None=True, doc="""
-        Bounds defined as (left, bottom, top, right) tuple.""")
+        Bounds defined as (left, bottom, right, top) tuple.""")
+
+
+class BoundsX(LinkedStream):
+    """
+    A stream representing the bounds of a box selection as an
+    tuple of the left and right coordinates.
+    """
+
+    boundsx = param.NumericTuple(default=None, constant=True, length=2,
+                                 allow_None=True, doc="""
+        Bounds defined as (left, right) tuple.""")
+
+
+class BoundsY(LinkedStream):
+    """
+    A stream representing the bounds of a box selection as an
+    tuple of the bottom and top coordinates.
+    """
+
+    boundsy = param.NumericTuple(default=None, constant=True, length=2,
+                                 allow_None=True, doc="""
+        Bounds defined as (bottom, top) tuple.""")
 
 
 class Selection1D(LinkedStream):
